#!/usr/bin/perl


BEGIN
{
    my ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    unshift(@INC, "$dirprg/../../../lib/");
}

our $VERSION = do {my @r = (q$Rev: 437 $ =~ /\d+/g); $r[0]};

use warnings;
use strict;
use IO::File;
use ParamParser;
my $O_PARAM;

MAIN:
{

    $O_PARAM = New ParamParser('GETOPTLONG',("infile=s", "quality_value=s","outfile=s"));
    $O_PARAM->SetUsage(my $usage = sub {&Usage();});
	$O_PARAM->AssertDefined("infile");

	my $quality_value = 20;
	if ($O_PARAM->IsDefined("quality_value"))
	{
		my $qual=$O_PARAM->Get("quality_value");
		if (($qual>0) && ($qual<=100))
		{
			$quality_value=$qual;
		}
		else
		{
			print STDERR "Warning : Invalid quality value $qual, default value used : $quality_value\n";
		}
	}

	$O_PARAM->AssertFileExists("infile");

	my $out_flux = *STDOUT;
	my $out;
	if ($O_PARAM->IsDefined("outfile"))
	{
		my $outfile = $O_PARAM->Get("outfile");
		my $out = new IO::File(">".$outfile) || Carp::croak ("$outfile");
		$out_flux = $out;
	}

	my $infile=$O_PARAM->Get("infile");
	my $in=new IO::File($infile) || Carp::croak ("$infile");
	my %h_sequence;
	my @a_sequence;
	my $compt=0;
	my $id;
	while (my $line=<$in>)
	{
		if ($line =~/>(.*)/)
		{
			$id=$1;
			$a_sequence[$compt] = $id;
			$compt++;
		}
		else
		{
			chomp $line;
			$h_sequence{$id} .= $line;
		}
	}

	for (my $i=0; $i<scalar(@a_sequence);$i++)
	{
		if (defined $a_sequence[$i])
		{
			my $id = $a_sequence[$i];
			if (defined $h_sequence{$id})
			{
				print $out_flux ">$id\n";
				my $seq=length($h_sequence{$id});
				for(my $i = 1; $i <$seq+1 ; $i++)
				{
					print $out_flux $quality_value;
					
					if (($i % 60) == 0)
					{
						print $out_flux "\n" 
					}
					else
					{
						print $out_flux " ";
					}
				}
				print $out_flux "\n";
			}
		}
	}
	$out_flux->close;
}

sub Usage
{
   print STDOUT <<END;

	Usage:

    $0 --infile <fasta file> --quality_value <int> --outfile <file>
	
	Mandatory :
		infile : the input fasta file
		
	Optional :	
		quality_value : an integer value for the quality (default : 20)
		outfile       : the path to the outfile


END
}

