#!/usr/bin/perl -I/usr/local/bioinfo/Xdom2.1/lib/

# Copyright INRA/CNRS
#
# sebastien.carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

	TGICL_assemble_filecon_add.pl
 
=head1 SYNOPSIS

 TGICL_assemble_filecon_add.pl --clusters_file<file> --outdirectory<dir> --paired_end_file<file>

=head1 DESCRIPTION

  TGICL_assemble_filecon_add.pl :
	Build the files .con for the cap3 step of TGICLplus

 TGICL_assemble_filecon_add.pl --clusters_file<file> --outdirectory<dir> --paired_end_file<file>
  Mandatory
	clusters_file : path to the clusters file
	outdirectory  : path to the repository in which save the .con file
	paired_ends_file : file with the identifiers of couple of transcripts known as paired ends (the ids of the couple have to be on the same line)
			exple : AL367179	AL367180
					MtRH15N17A1	MtRH15N17N1

=head1 AUTHORS
 
 Marion Verdenaud	marion.verdenaud@toulouse.inra.fr
 Sebastien Carrere  sebastien.carrere@toulouse.inra.fr
 Jerome Gouzy	    Jerome.Gouzy@toulouse.inra.fr

=cut



use strict;
use IO::File;

use ParamParser;
my $O_PARAM;

MAIN:
{
	$O_PARAM = New ParamParser("GETOPTLONG",("clusters_file=s","outdirectory=s","paired_end_file=s"));
	$O_PARAM->SetUsage(my $usage = sub {&Usage();});
	#check the inputs
	my $file = $O_PARAM->Get('clusters_file');
	my $rep = $O_PARAM->Get('outdirectory');
	my $clusters_file= new IO::File($file)||Carp::croak("can't open the file : $file");
	my $paired_end_file = $O_PARAM->Get('paired_end_file');
	
	my $paired_end_file = new IO::File($paired_end_file)||Carp::croak("can't open the paired end file : $paired_end_file");
	my %h_treated;
	my %h_paired_ends;
	while (my $ligne=<$paired_end_file>)
	{
		
		my @a_paired = split ("\t",$ligne);
		my $pair1=$a_paired[0];
		my $pair2=$a_paired[1];
		
		chomp $pair1;
		chomp $pair2;
		
		$h_paired_ends{$pair1}=$pair2;
		$h_paired_ends{$pair2}=$pair1;
	}
	$paired_end_file->close;

	
	my $con_file;
	my $id;
	my @a_con_file;
	my %h_present_file=();
	
	while (my $ligne=<$clusters_file>)
	{
		if ($ligne =~/^>(CL\d+)/)
		{
			$id=$1;
			
		}
		else
		{
			my %h_component;
			my @a_line=split ("\t",$ligne);
			foreach my $est (@a_line)
			{
				$h_component{$est}="" if ($est !~ /\s+/);
			}
			$h_present_file{$id}=\%h_component;
		}
	}
	$clusters_file->close;

	my @a_repository = `find -name "$rep"`;

	#build the .con file, linked to each cluster
	foreach my $cluster (keys %h_present_file)
	{
		my $con_data="";
	
		my $rh_component = $h_present_file{$cluster};
		my %h_component = %$rh_component;
		foreach my $est (keys %h_component)
		{
			if ((defined $h_paired_ends{$est}))
			{
				my $pair=$h_paired_ends{$est};
				if ((defined $h_component{$pair}) && (! defined $h_treated{$est}))
				{
					$con_data .= "$est\t".$h_paired_ends{$est}."\t100\t6000\n";
# 					print $con_file "$est\t".$h_paired_ends{$est}."\t100\t6000\n";
					$h_treated{$est}="";
					$h_treated{$pair}="";
				}
			}
		}

		if ($con_data ne "")
		{
			$con_file = new IO::File(">/tmp/$cluster.con")||Carp::croak("can't open the con file $id");
			print $con_file $con_data;
			$con_file->close;

			foreach my $r (@a_repository)
			{
				chomp $r;
				system("cp /tmp/$cluster.con $r");
				push(@a_con_file,"$r/$cluster.con");
			}
			unlink ("/tmp/$cluster.con");	
		}
	}
	
# 	foreach my $file (@a_con_file)
# 	{
# # 		print STDOUT $file."\n";
# 		unlink $file if (-z $file);
# 	}
}


sub Usage
{
print STDOUT <<END;

Usage:
     $0 --clusters_file<file> --outdirectory<dir> --paired_end_file<file>
  
    Mandatory
        clusters_file    : path to the clusters file
        outdirectory     : path to the repository in which save the .con file
        paired_ends_file : file with the identifiers of couple of transcripts known as paired ends (the ids of the couple have to be on the same line)
            exple : AL367179	AL367180
					MtRH15N17A1	MtRH15N17N1

END

}
