#!/usr/bin/perl


# Copyright INRA

# Sebastien.Carrere@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide
# a code generator framework for BioMoby web-services.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME

install.pl - TGICL++ Install/Configuration script

=head1 SYNOPSIS

install.pl [--paraloop_repository_path <dir>] [--verbose]

=head1 DESCRIPTION

When you install tgicl environnement for the first time you have to configure it.
The script install.pl will modify cfg file to tune tgicl according to your install dir. IT also check the that all the require module are installed.


% tcsh
% ./install.pl [--paraloop_repository_path <dir>]


    Optional
		--paraloop_repository_path       : path to the repository where Paraloop is installed, if $PARALOOP it's not defined in the path
		--verbose with specific timeout)


=head1 AUTHORS

Marion.Verdenaud


=head1 SUBROUTINES

=cut

use strict;
my ($dirprg, $nameprg);
use Term::ANSIColor;
use FindBin;

BEGIN
{

    ($dirprg, $nameprg) = $0 =~ /(.+)\/(.+)/;
    
    $dirprg = '.' if (! (defined $dirprg) || ($dirprg eq ''));
    
    if (-d "$dirprg/lib")
    {
        unshift(@INC, "$dirprg/lib");
    }
    if (-d "$dirprg/../../lib")
    {
        unshift(@INC, "$dirprg/../../lib");
    }

}

$ENV{'PATH'}=$FindBin::Bin.'/bin:'.$ENV{'PATH'};

use ParamParser;
use File::Which qw(which);
use Cwd 'abs_path';

my $O_PARAM;
my @A_MANDATORY_BINARIES = ('mv','rm','find','cat','tar','gzip','gunzip') ;
$dirprg = abs_path($dirprg);

MAIN:
{

    $O_PARAM = New ParamParser('GETOPTLONG', ("paraloop_repository_path=s","verbose","h"));
	$O_PARAM->SetUsage(my $usage = sub {&Usage();});

	if ($O_PARAM->IsDefined('h'))
	{
		$O_PARAM->Usage();
	}

	$O_PARAM->SetUnlessDefined('unix_path',abs_path($dirprg));
	my $unix_path=$O_PARAM->Get('unix_path');

	if ($O_PARAM->IsDefined('paraloop_repository_path'))
	{
		$O_PARAM->AssertDirExists('paraloop_repository_path');
	}
	else
	{
		my $paraloop_path=`printenv PARALOOP`;
		chomp $paraloop_path;
		if ($paraloop_path ne "")
		{
			$O_PARAM->Set('paraloop_repository_path',$paraloop_path);
		}
		else
		{
			print STDERR color 'red';
			print STDERR "The parameter paraloop_repository_path  or the environment variable PARALOOP have to be defined\n";
			print STDERR color 'reset';
			exit;
		}
	}

	print STDERR "\nunix_path $unix_path\n";

	my $rh_mask_binaries = &CheckConfig();

	my $unix_file_mask = $unix_path . '/cfg/unix_cmd.cfg.mask';
    die("ERROR: $unix_file_mask not found") if (!-e $unix_file_mask);
    my $unix_file = $unix_path . '/cfg/unix_cmd.cfg';
    print STDERR "Substitute $unix_file_mask\n";
    &Substitute($unix_file_mask, $unix_file,$rh_mask_binaries);

# 	my $pgm_file_mask = $unix_path . '/bin/tgicl_cluster.psx.mask';
#     die("ERROR: $pgm_file_mask not found") if (!-e $pgm_file_mask);
#     my $pgm_file = $unix_path . '/bin/tgicl_cluster.psx';
#     print STDERR "Substitute $pgm_file_mask\n";
#     &Substitute($pgm_file_mask, $pgm_file,$rh_mask_binaries);

}

=head2 procedure CheckConfig

 Title        : CheckConfig

=cut

sub CheckConfig
{

    my @a_mandatory_modules = (
								'IO::File',			'ParamParser',
								'FindBin',			'FileHandle',
								'Socket',			'BerkeleyDB',
								'Getopt::Std',
                             );

	my @a_tgicl_binaries =('tgicl_asm.psx','tgicl_cluster.psx', 'zmsort', 'tclust', 'sclust','formatdb', 'mgblast', 'cdbfasta', 'cdbyank', 'psx','mdust','tgicl_assemble_filecon_add.pl','tgicl_clustering_pairedends_add.pl','create_quality_file.pl','split_fasta.pl');

    my $config_ok = 1;

    my %h_mask_binaries = ();

	$h_mask_binaries{'#mask_unix#'} = $O_PARAM->Get('unix_path');

    foreach my $mandatory_binary (@A_MANDATORY_BINARIES)
    {
        my $exe_path = which($mandatory_binary);

	$h_mask_binaries{'#mask_' . $mandatory_binary . '#'} = $exe_path;
	
        if (!defined($exe_path) || ($exe_path eq ''))
        {
            print STDERR color 'red';
            print STDERR "\nERROR CONFIGURATION: $mandatory_binary binary can not be found on your system\n\n";
            print STDERR color 'reset';
			$config_ok = 0;
		}
		else
		{
			print STDERR color 'bold green';
			print STDERR "[OK] $mandatory_binary = $exe_path\n" if ($O_PARAM->IsDefined('verbose'));
			print STDERR color 'reset';
		}
	}

    foreach my $mandatory_module (@a_mandatory_modules)
    {
        eval "use $mandatory_module;";
        if ($@)
        {
            print STDERR color 'red';
            print STDERR "\nERROR CONFIGURATION: $mandatory_module can not be found on your system: $@\nYou can find it at CPAN URL: http://search.cpan.org/search?mode=module&query=$mandatory_module\n\n";
	    print STDERR color 'reset';
            $config_ok = 0;
        }
        else
        {
            print STDERR color 'bold green';
            print STDERR "[OK] $mandatory_module\n" if ($O_PARAM->IsDefined('verbose'));
            print STDERR color 'reset'; 
        }
    }
    
	#verif paraloop installed
	my $paraloop_rep = $O_PARAM->Get('paraloop_repository_path');
	my $paraloop_path=$paraloop_rep."/bin/paraloop.pl";
	my $exe_path = which($paraloop_path);
	$h_mask_binaries{'#mask_paraloop#'} = $paraloop_rep;
	if (!defined($paraloop_path) || ($paraloop_path eq ''))
	{
		print STDERR color 'red';
		print STDERR "\nERROR CONFIGURATION: $paraloop_path binary can not be found on your system\n\n";
		print STDERR color 'reset';
		$config_ok = 0;
	}
	else
	{
		print STDERR color 'bold green';
		print STDERR "[OK] paraloop.pl = $paraloop_path\n" if ($O_PARAM->IsDefined('verbose'));
		print STDERR color 'reset';
	}


	foreach my $tgicl_binary (@a_tgicl_binaries)
	{
		my $r=`which $tgicl_binary`;
		if (-e "$dirprg/bin/$tgicl_binary" || $r !~ m/not found/si || $r =~ m/\/$tgicl_binary/s)
		{
			print STDERR color 'bold green';
            print STDERR "[OK] $tgicl_binary = $dirprg/bin/$tgicl_binary\n" if ($O_PARAM->IsDefined('verbose'));
            print STDERR color 'reset';
		}
		else
		{
			print STDERR color 'red';
   			print STDERR "Error: cannot find the executable '$tgicl_binary' anywhere in shell's path:\n".`echo \$PATH`."or in the bin reposotory : $dirprg/bin/\n";
            print STDERR color 'reset';
            $config_ok = 0;
		}
	}

    if ($config_ok == 0)
    {
	print STDERR color 'bold red';
    	print STDERR ("\n\nPRE-REQUISITE MODULES/BINARIES ARE MISSING. CONFIGURATION FAILED\n\n\n");
	print STDERR color 'reset';
	exit;
    }
    else
    {
	print STDERR color 'bold green';
    	print STDERR "\n\nCONFIGURATION IS OK\n\n\n";
	print STDERR color 'reset';
    }

    return \%h_mask_binaries;

}


=head2 procedure Substitute

 Title        : Substitute

=cut

sub Substitute
{
    my ($in, $out, $rh_mask_binaries) = (@_);

    my $in_fh  = new IO::File($in);
    my $out_fh = new IO::File(">$out");
    die("$!: $in")  if (!defined $in_fh);
    die("$!: $out") if (!defined $out_fh);

    while (my $line = <$in_fh>)
    {
        chomp $line;
        if (defined $rh_mask_binaries)
		{
			foreach my $mask_binary (keys %{$rh_mask_binaries})
    		{
				$line =~ s/$mask_binary/$rh_mask_binaries->{$mask_binary}/g;
			}
		}

        print $out_fh $line . "\n";
    }

    $in_fh->close;
    $out_fh->close;
}


sub Usage
{
	 print STDOUT <<END;

Usage:


     $0 [--paraloop_repository_path <dir>] [--verbose] [--h]

    Optional
		--paraloop_repository_path      : path to the repository where Paraloop is installed, if \$PARALOOP it's not defined in the path
		--h                             : display the usage
		--verbose
END
}
